import cv2
import numpy as np

import cmac_gdal_io as io



def convert_image_to_vector(input_image_buffer, num_gridcell_rows, num_gridcell_cols,
                            nodata_value):

    # Initializations.
    num_grid_image_rows = 0
    num_grid_image_cols = 0
    num_img_rows = input_image_buffer.shape[0]
    num_img_cols = input_image_buffer.shape[1]

    input_imagedata_list = []
    

    # Gridcell "center".
    gc_mid_index = (num_gridcell_rows * num_gridcell_cols) // 2

    # MAIN GRIDCELL EXTRACTION LOOP(S)
    for i in range(0, num_img_rows, num_gridcell_rows):

        # Define the gridcell area within the image.
        if (i + num_gridcell_rows) > num_img_rows:
            rw_height = num_img_rows - i
        else:
            rw_height = num_gridcell_rows

        for j in range(0, num_img_cols, num_gridcell_cols):
            if (j + num_gridcell_cols) > num_img_cols:
                rw_width = num_img_cols - j
            else:
                rw_width = num_gridcell_cols

            gridcell_buff = input_image_buffer[i:i+rw_height, j:j+rw_width].flatten()

            # TODO:  Revisit gridcell filling in C++ and Python CMAC versions.
            if gridcell_buff[0] != nodata_value:
                input_imagedata_list.append(gridcell_buff[0])
            elif gridcell_buff[gc_mid_index] != nodata_value:
                input_imagedata_list.append(gridcell_buff[gc_mid_index])
            else:
                input_imagedata_list.append(nodata_value)

            # Count the number of grid image columns only for the first row.
            if i == 0:
                num_grid_image_cols += 1

        # Count the number of grid image rows.
        num_grid_image_rows += 1

    input_image_vector = np.array(input_imagedata_list).astype(np.uint16)

    return num_grid_image_rows, num_grid_image_cols, input_image_vector




def convert_vector_to_grid(input_image_vector, num_grid_image_rows, num_grid_image_cols,
                           dtype=np.uint16):

    # Initialization
    grid_image_buff \
        = np.reshape(input_image_vector, (num_grid_image_rows, num_grid_image_cols))

    return grid_image_buff




def convert_grid_to_vector(input_grid_image):

    # Initializations.
    grid_vector = input_grid_image.flatten()

    return grid_vector




def convert_image_to_grid(input_raster_band, num_gridcell_rows,
                          num_gridcell_cols, nodata_value):

    # Convert the image to a vector/list representation.
    num_grid_image_rows, num_grid_image_cols, input_image_vector \
        = convert_image_to_vector(input_raster_band, num_gridcell_rows, num_gridcell_cols,
                                  nodata_value)
    if num_grid_image_rows < 0:
        err_msg = 'ERROR: could not convert image to vector/list representation'
        print(err_msg)
        return None

    # Convert the vector to a grid image representation.
    grid_image_buff = convert_vector_to_grid(input_image_vector, num_grid_image_rows,
                                             num_grid_image_cols)

    return grid_image_buff





def convert_vector_to_image(smoothed_vec, num_img_rows,
                            num_img_cols, num_gridcell_rows,
                            num_gridcell_cols):

    # Initializations.
    num_grid_datapoints = smoothed_vec.shape[0]
    
    # Create a full-sized raster image buffer
    output_image_raster = np.zeros((num_img_rows, num_img_cols),
                                   dtype=np.uint16)

    rw_height = rw_width = 0
    smoothed_vec_index = 0
    for i in range(0, num_img_rows, num_gridcell_rows):
        
        if (i % 100) == 0:
            print(f'Processing image row {i + 1}...\r')
        # Determine how much of the gridcell to write to the
        # raster image buffer.
        if (i + num_gridcell_rows) > num_img_rows:
            rw_height = num_image_rows - i
        else:
            rw_height = num_gridcell_rows

        for j in range(0, num_img_cols, num_gridcell_cols):

            if (j + num_gridcell_cols) > num_img_cols:
                rw_width = num_img_cols - j
            else:
                rw_width = num_gridcell_cols

            # Populate the gridcell buffer with the current grid
            # image pixel value.
            gc_pixval = smoothed_vec[smoothed_vec_index]

            # Write the populated gridcell to its position in the
            # full-sized raster.
            output_image_raster[i:i+rw_height, j:j+rw_width] = gc_pixval

            # Increment to the next grid "pixel" value.
            smoothed_vec_index += 1

    print('')

    return output_image_raster



