import numpy as np


def get_valid_data(gridcell_data,
                   nodata_value):

    return ((gridcell_data != nodata_value) \
        & (~np.isnan(gridcell_data)))




# If we're considering "descriptive" statistics to
# first order, then these functions could be replaced
# by a single function.

def get_gridcell_minimum(gridcell_data,
                         nodata_value):

    # Just consider 'valid' image data.
    valid_data = get_valid_data(gridcell_data,
                                nodata_value)
    if not np.any(valid_data):
        return nodata_value

    return np.min(gridcell_data[valid_data])




def get_gridcell_maximum(gridcell_data,
                         nodata_value):

    # Just consider 'valid' data.
    valid_data = get_valid_data(gridcell_data,
                                nodata_value)
    if not np.any(valid_data):
        return nodata_value

    return np.max(gridcell_data[valid_data])




def get_gridcell_mean(gridcell_data,
                      nodata_value):

    # Just consider 'valid' data.
    valid_data = get_valid_data(gridcell_data,
                                nodata_value)
    if not np.any(valid_data):
        return nodata_value

    return np.mean(gridcell_data[valid_data])




def get_gridcell_std(gridcell_data,
                     nodata_value):

    # Just consider 'valid' data.
    valid_data = get_valid_data(gridcell_data,
                                nodata_value)
    if not np.any(valid_data):
        return nodata_value

    return np.std(gridcell_data[valid_data])




def get_gridcell_median(gridcell_data,
                        nodata_value):

    # Just consider 'valid' data.
    valid_data = get_valid_data(gridcell_data,
                                nodata_value)
    if not np.any(valid_data):
        return nodata_value

    return np.median(gridcell_data[valid_data])
