import os
from pathlib import Path

from osgeo import gdalconst

import cmac_gdal_io as io



def load_toa_images(toa_image_filenames_list,
                    bands_to_load_list):

    # Sanity check.
    num_bands_to_load = len(toa_image_filenames_list)
    num_bands_to_process = len(bands_to_load_list)
    if num_bands_to_load != num_bands_to_process:
        err_msg = f'ERROR: {num_bands_to_load} to load, ' \
            + f'but {num_bands_to_process} need to be ' \
            + 'processed'
        print(err_msg)
        return []

    toa_datasets_list = []
    num_bands_loaded = 0

    # MAIN LOADING LOOP
    for i in range(num_bands_to_load):

        print(f'Loading band {bands_to_load_list[i]} ...')
        if not Path(toa_image_filenames_list[i]).exists():
            err_msg = f'ERROR: TOA image path "{toa_image_filenames_list[i]}" ' \
                + 'not found'
            print(err_msg)
            continue

        curr_dataset = io.open_dataset(toa_image_filenames_list[i],
                                       gdalconst.GA_ReadOnly)
        if not curr_dataset:
            err_msg = f'ERROR: cannot load "{toa_image_filenames_list[i]}"'
            print(err_msg)
            continue

        # Add this open dataset to the list.
        toa_datasets_list.append(curr_dataset)
        num_bands_loaded += 1

    # END MAIN LOADING LOOP

    # Final check -- how many bands did we load? If it's not all,
    # consider that an error. If this occurs, close the dataset(s)
    # that did get loaded and return an empty list.
    if num_bands_loaded != num_bands_to_load:
        err_msg = f'ERROR: {num_bands_loaded} were loaded, but ' \
            + f'{num_bands_to_load} needed to be loaded'
        print(err_msg)
        for k in range(num_bands_loaded):
            toa_datasets_list[k] = None
        return []

    return toa_datasets_list






