from pathlib import Path

import numpy as np

import cmac_gdal_io as io
import cmac_utility as util



def create_nodata_mask_buffer(input_image_buffer,
                              nodata_value):
    
    # Allocate the data buffer.
    buff_ysize = input_image_buffer.shape[0]
    buff_xsize = input_image_buffer.shape[1]

    mask_image_buffer = (input_image_buffer != nodata_value).astype(input_image_buffer.dtype)

    return mask_image_buffer




def create_nodata_mask_images(toa_datasets_list,
                              bands_to_process_list,
                              nodata_value,
                              GTiff_mask_filenames_list,
                              product_mask_filenames_list):

    num_toa_datasets = len(toa_datasets_list)

    for index, ds in enumerate(toa_datasets_list):
    #for i in range(num_toa_datasets):

       # First, make sure a mask file hasn't already been
       # created for this band.
       if Path(GTiff_mask_filenames_list[index]).exists():
           print(f'Band {bands_to_process_list[index]} has already been ' \
               + 'created, skipping')
           continue

       print(f'Creating band {bands_to_process_list[index]} mask image...')
       
       # Get the geotransform and projection information.
       geotransform_info = io.get_geotransform_info(ds)
       proj_info_string = io.get_projection_info(ds)

       # Read the image data.
       image_data_buffer = io.read_image_from_open_dataset(ds)

       # Create the mask image.
       mask_data_buffer = create_nodata_mask_buffer(image_data_buffer,
                                                    nodata_value)

       # Write the mask image to file.
       num_image_rows = mask_data_buffer.shape[0]
       num_image_cols = mask_data_buffer.shape[1]
       status = io.write_image_to_file(mask_data_buffer, num_image_rows,
                                       num_image_cols, geotransform_info,
                                       proj_info_string, GTiff_mask_filenames_list[index],
                                       product_mask_filenames_list[index])

       if status < 0:
           err_msg = 'ERROR: could not create new mask image file for ' \
               + f'"{GTiff_mask_filenames_list[i]}" and/or ' \
               + f'"{product_mask_filenames_list[i]}"'
           print(err_msg)

    return status