from osgeo import gdalconst
import numpy as np

import cmac_gdal_io as io


def upsample_by_2(input_filename, geotransform_info,
                proj_info_string, output_filename_source,
                output_filename_product):

    # Initializations.
    final_status = 0

    # Read the image.
    input_data = io.read_image_from_file(input_filename)
    if input_data is None:
        err_msg = f'ERROR: cannot load input image "{input_filename}"'
        print(err_msg)
        return -1

    # Determine the output image dimensions.
    in_num_rows = input_data.shape[0]
    in_num_cols = input_data.shape[1]
    out_num_rows = 2 * in_num_rows
    out_num_cols = 2 * out_num_cols

    print(f'Resampling "{input_filename}" to 2x spatial resolution...')
    output_data = np.zeros((out_num_rows, out_num_cols), dtype=input_data.dtype)
    for i in range(in_num_rows):
        in_row_offset = i * in_num_cols

        for j in range(in_num_cols):
            in_pix_offset = in_row_offset + j
            inputPix = input_data[in_pix_offset]

            for k in range(2):
                out_row_offset = ((2 * i) + k) * out_num_cols
                for l in range(2):
                    out_pix_offset = out_row_offset + ((2 * j) + l)
                    output_data[out_pix_offset] = inputPix

    print('Saving (re)sampled image...')
    status = io.write_image_to_file(output_data, out_num_rows, out_num_cols,
                                 geotransform_info, proj_info_string,
                                 output_filename_source, output_filename_product)
    if status < 0:
        err_msg = 'ERROR: could not save resampled image data'
        print(err_msg)
        final_status = -1

    return final_status




def downsample_by_2(input_filename, geotransform_info,
                  proj_info_string, output_filename_source,
                  output_filename_product):

    # Initializations.
    final_status = 0

    # Read the image.
    input_data = io.read_image_from_file(input_filename)
    if input_data is None:
        err_msg = f'ERROR: cannot load input image "{input_filename}"'
        print(err_msg)
        return -1

    # Determine the output image dimensions.
    in_num_rows = input_data.shape[0]
    in_num_cols = input_data.shape[1]
    out_num_rows = (in_num_rows // 2)
    out_num_cols = (in_num_cols // 2)

    # Flatten the input image.
    input_data_flat = input_data.flatten()

    # Grab every other pixel.
    print(f'Resampling "{input_filename}" to 0.5x spatial resolution...')
    output_data_flat = np.zeros((out_num_rows*out_num_cols), dtype=input_data.dtype)
    output_index = 0
    for i in range(0, in_num_rows, 2):
        if (i % 100) == 0:
            print(f'Processing input row {i+1}...\r')
        for j in range(0, in_num_cols, 2):
            output_data_flat[output_index] = input_data[i, j]
            output_index += 1
    print('')

    # Reshape into a 2D "image".
    output_data = np.reshape(output_data_flat, (out_num_rows, out_num_cols))

    print('Saving (re)sampled image...')
    status = io.write_image_to_file(output_data, out_num_rows, out_num_cols,
                                    geotransform_info, proj_info_string,
                                    output_filename_source, output_filename_product)
    if status < 0:
        err_msg = 'ERROR: could not save resampled image data'
        print(err_msg)
        final_status = -1

    return final_status

